﻿using Infragistics.Win.UltraWinGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace _147466
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            DataTable dtG1 = new DataTable();

            dtG1.Columns.Add("id", typeof(int)).AutoIncrement = true;
            dtG1.Columns.Add("col1", typeof(string));
            dtG1.Columns.Add("col2", typeof(string));
            dtG1.Columns.Add("col3", typeof(string));

            DataTable dtG2 = dtG1.Clone();

            for (int i = 0; i < 10; i++)
            {
                string textG1 = string.Format("G1 Row {0}", i);
                dtG1.Rows.Add(null, textG1, textG1, textG1);
                string textG2 = string.Format("G2 Row {0}", i);
                dtG2.Rows.Add(null, textG2, textG2, textG2);
            }

            ultraGrid1.DataSource = dtG1;
            ultraGrid2.DataSource = dtG2;

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            AssignDragDropForUltraGrid.AttacheDragDropToUltraGrid(ultraGrid1);
            AssignDragDropForUltraGrid.AttacheDragDropToUltraGrid(ultraGrid2);
        }


        private void ultraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.Override.CellClickAction = CellClickAction.RowSelect;
            e.Layout.Override.SelectTypeRow = SelectType.ExtendedAutoDrag;
        }
    }


    public static class DataTableEX
    {
        public static bool AddCloning(this DataTable table, DataRow row, int atIndex = -1)
        {
            if (atIndex < 0)
                atIndex = table.Rows.Count;
            DataRow addRow = table.NewRow();
            foreach(DataColumn column  in row.Table.Columns)
            {
                string key = column.ColumnName;
                
                if (!table.Columns.Contains(key))
                    continue;
                
                if (table.Columns[key].AutoIncrement || 
                    !string.IsNullOrEmpty(table.Columns[key].Expression))
                    continue;

                addRow[key] = row[key];
            }

            DataColumn[] errors = addRow.GetColumnsInError();
            if (errors == null || errors.Length == 0)
            {
                table.Rows.InsertAt(addRow, atIndex);
                return true;
            }
            else
                return false;

        }

        public static bool AddClonigRange(this DataTable table, DataRow[] rows, int atIndex = -1)
        {
            bool addingRes = true;
            for (int i = rows.Length -1; i >= 0 ; i--)
            {
               addingRes = addingRes &  table.AddCloning(rows[i], atIndex);
            }
            return addingRes;
        }

    }

    public  class AssignDragDropForUltraGrid
    {
       public static void  AttacheDragDropToUltraGrid (UltraGrid grid)
       {
           grid.AllowDrop = true;
           // starting point if drag should start 
           grid.SelectionDrag += grid_SelectionDrag;
           grid.DragDrop += grid_DragDrop;
           grid.DragEnter += grid_DragEnter;
           grid.DragLeave += grid_DragLeave;
           grid.DragOver += grid_DragOver;
           grid.QueryContinueDrag += grid_QueryContinueDrag;
            
       }

       static void grid_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
       {
          // throw new NotImplementedException();
       }

       static void grid_DragOver(object sender, DragEventArgs e)
       {
           GridDataHolder data = e.Data.GetData(typeof(GridDataHolder)) as GridDataHolder;
           if (data == null)
               e.Effect = DragDropEffects.None;
           if (data.GirdPtr.Equals(((Control)sender).Handle))
               e.Effect = DragDropEffects.None;
       }

       static void grid_DragLeave(object sender, EventArgs e)
       {
          // throw new NotImplementedException();
       }

       static void grid_DragEnter(object sender, DragEventArgs e)
       {
           GridDataHolder data = e.Data.GetData(typeof(GridDataHolder)) as GridDataHolder;
           // no data
           if (data == null)
               e.Effect = DragDropEffects.None;
           // drag is performed on the same component 
           if (data.GirdPtr.Equals(((Control)sender).Handle))
               e.Effect = DragDropEffects.None;

           e.Effect = DragDropEffects.All;
       }

       static void grid_DragDrop(object sender, DragEventArgs e)
       {
           if (e.Effect == DragDropEffects.None)
               return;
           UltraGrid grid = (UltraGrid) sender;
           DataTable source = (DataTable)grid.DataSource;
           GridDataHolder data = e.Data.GetData(typeof(GridDataHolder)) as GridDataHolder;
           if (data != null)
           {
               source.AddClonigRange(data.Rows);
           }
       }

       static void grid_SelectionDrag(object sender, CancelEventArgs e)
       {
           UltraGrid grid = (UltraGrid)sender;
           if (grid.Selected.Rows == null || grid.Selected.Rows.Count == 0)
               return;
           var gridRows = grid.Selected.Rows.Cast<UltraGridRow>().ToArray();
           DataRow[] rows = new DataRow[gridRows.Length];
           for (int i = 0; i < gridRows.Length; i++)
           {
               rows[i] = ((DataRowView)gridRows[i].ListObject).Row;
               
           }
           grid.DoDragDrop(new GridDataHolder(grid, rows), DragDropEffects.All);
       }

        class GridDataHolder
        {
            DataRow[] rows;
            public DataRow[] Rows
            {
                get { return rows; }
            }

            IntPtr girdPtr;
            public IntPtr GirdPtr
            {
                get { return girdPtr; }
            }

            public GridDataHolder(Control grid, DataRow[] rows)
            {
                this.girdPtr = grid.Handle;
                this.rows = rows;
            }


        }

    }
}
